package crafttweaker.mc1120.player;

import crafttweaker.CraftTweakerAPI;
import crafttweaker.api.chat.IChatMessage;
import crafttweaker.api.data.IData;
import crafttweaker.api.item.IItemStack;
import crafttweaker.api.minecraft.CraftTweakerMC;
import crafttweaker.api.player.IPlayer;
import crafttweaker.mc1120.CraftTweaker;
import crafttweaker.mc1120.data.NBTConverter;
import crafttweaker.mc1120.network.*;
import net.minecraft.command.*;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.*;
import net.minecraft.init.SoundEvents;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.text.*;

/**
 * @author Stan
 */
public class MCPlayer implements IPlayer {
    
    private final EntityPlayer player;
    
    public MCPlayer(EntityPlayer player) {
        this.player = player;
    }
    
    public EntityPlayer getInternal() {
        return player;
    }
    
    @Override
    public String getId() {
        return player.func_110124_au().toString();
    }
    
    @Override
    public String getName() {
        return player.func_70005_c_();
    }
    
    @Override
    public IData getData() {
        return NBTConverter.from(player.getEntityData(), true);
    }
    
    @Override
    public int getXP() {
        return player.field_71068_ca;
    }
    
    @Override
    public void setXP(int xp) {
        player.func_82242_a(-player.field_71068_ca);
        player.func_82242_a(xp);
    }
    
    @Override
    public void removeXP(int xp) {
        player.func_71023_q(-xp);
    }
    
    @Override
    public void update(IData data) {
        NBTConverter.updateMap(player.getEntityData(), data);
    }
    
    @Override
    public void sendChat(IChatMessage message) {
        Object internal = message.getInternal();
        if(!(internal instanceof ITextComponent)) {
            CraftTweakerAPI.logError("not a valid chat message");
            return;
        }
        player.func_145747_a((ITextComponent) internal);
    }
    
    @Override
    public void sendChat(String message) {
        player.func_145747_a(new TextComponentString(message));
    }
    
    @Override
    public int getHotbarSize() {
        return 9;
    }
    
    @Override
    public IItemStack getHotbarStack(int i) {
        return i < 0 || i >= 9 ? null : CraftTweakerMC.getIItemStack(player.field_71071_by.func_70301_a(i));
    }
    
    @Override
    public int getInventorySize() {
        return player.field_71071_by.func_70302_i_();
    }
    
    @Override
    public IItemStack getInventoryStack(int i) {
        return CraftTweakerMC.getIItemStack(player.field_71071_by.func_70301_a(i));
    }
    
    @Override
    public IItemStack getCurrentItem() {
        return CraftTweakerMC.getIItemStack(player.field_71071_by.func_70448_g());
    }
    
    @Override
    public boolean isCreative() {
        return player.field_71075_bZ.field_75098_d;
    }
    
    @Override
    public boolean isAdventure() {
        return !player.field_71075_bZ.field_75099_e;
    }
    
    @Override
    public void openBrowser(String url) {
        if(player instanceof EntityPlayerMP) {
            CraftTweaker.NETWORK.sendTo(new MessageOpenBrowser(url), (EntityPlayerMP) player);
        }
    }
    
    @Override
    public void copyToClipboard(String value) {
        if(player instanceof EntityPlayerMP) {
            CraftTweaker.NETWORK.sendTo(new MessageCopyClipboard(value), (EntityPlayerMP) player);
        }
    }
    
    @Override
    public boolean equals(Object other) {
        return other.getClass() == this.getClass() && ((MCPlayer) other).player == player;
        
    }
    
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 23 * hash + (this.player != null ? this.player.hashCode() : 0);
        return hash;
    }
    
    @Override
    public void give(IItemStack stack) {
        ItemStack itemstack = CraftTweakerMC.getItemStack(stack).func_77946_l();
        boolean flag = player.field_71071_by.func_70441_a(itemstack);
        
        if(flag) {
            player.field_70170_p.func_184148_a(null, player.field_70165_t, player.field_70163_u, player.field_70161_v, SoundEvents.field_187638_cR, SoundCategory.PLAYERS, 0.2F, ((player.func_70681_au().nextFloat() - player.func_70681_au().nextFloat()) * 0.7F + 1.0F) * 2.0F);
            player.field_71069_bz.func_75142_b();
        }
        
        if(flag && itemstack.func_190926_b()) {
            itemstack.func_190920_e(1);
            EntityItem entityitem1 = player.func_71019_a(itemstack, false);
            
            if(entityitem1 != null) {
                entityitem1.func_174870_v();
            }
        } else {
            EntityItem entityitem = player.func_71019_a(itemstack, false);
            
            if(entityitem != null) {
                entityitem.func_174868_q();
                entityitem.func_145797_a(player.func_70005_c_());
            }
        }
    }
    
    @Override
    public double getX() {
        return player.field_70165_t;
    }
    
    @Override
    public double getY() {
        return player.field_70163_u;
    }
    
    @Override
    public double getZ() {
        return player.field_70161_v;
    }
}
